; Public domain tool to fetch the load arguments of a MEMDISK
; (virtual disk, created by SYSLINUX MEMDISK tool in int 15h
; memory space after booting with SYSLINUX / ISOLINUX / ...
; to simplyfy booting non-Linux operating systems). Based on
; the IFMEMDSK tool. By Eric Auer 9/2004.

; Returned Errorlevel: (without / with Win32 running)
; 0 / 16 - 386 or better CPU but no MEMDISK
; 1 / 17 - A: is a MEMDISK
; 2 / 18 - B: is a MEMDISK
; 3 / 19 - C:, or to be more exact, the first harddisk, BIOS disk
;     number 80h, is a MEMDISK
; 
; 254 - the CPU is older than an 80286
; 255 - the CPU is an 80286 only, so no MEMDISK possible


%imacro	FLAGCHECK 0
	pushf		; save
	push ax		; test value
	popf		; try to set flags
	pushf		; check what happened
	pop ax		; test results
	popf		; restore
%endmacro


	org 100h	; a .COM file

start:	mov ah,9	; string$ output
	mov dx,hello	; hello message
	int 21h		; DOS API

	xor bp,bp	; errorlevel
	dec bp
	dec bp

	xor ax,ax	; try to zero all flags
	mov bx,0f000h	; bit mask for high bits
	FLAGCHECK
	and ax,bx	; high bits...
	cmp ax,bx	; ...stuck to 1? 80186 or older!
	jz known	; *** is186: errorlevel 254, PC XT
	inc bp		; next assumption: 286
	mov ax,bx	; try to set all high bits
	FLAGCHECK
	test ax,bx	; all stuck to 0? 80286!
	jz known	; *** is286: errorlevel 255, PC AT but no 386+
	inc bp		; next: 386, but no memdisk found yet.

is386:	inc bp		; assumption: errorlevel 1, A: is a memdisk
	mov dl,0	; A:
	call memdiskcheck
	jc known	; only return first match
	inc bp		; assumption: errorlevel 2, B: is a memdisk
	mov dl,1	; B:
	call memdiskcheck
	jc known	; only return first match
	inc bp		; assumption: errorlevel 3, "C:" is a memdisk
	mov dl,80h	; "C:"
	call memdiskcheck
	jc known	; only return first match
noABCmemdisk:
	xor bp,bp	; errorlevel 0, no memdisk but at least a 386.

known:	mov ax,bp	; fetch errorlevel
	cbw		; sign-extend AL to AX
	add ax,ax
	mov bx,ax
	mov dx,[cs:stringtable+bx]	; fetch appropriate message
; - done:
	mov ah,9	; string$ output ds:dx
	int 21h		; DOS API
	mov dx,memdisk	; write " MEMDISK." and carriage return, line break
	mov ah,9
	int 21h
	mov ax,bp	; fetch errorlevel	
	call WinCheck	; check for Win32, add 16 to errorlevel if found
	mov ah,4ch	; leave program, errorlevel in AL
	int 21h		; DOS API


memdiskcheck:
	mov eax,454d0800h	; magic1 + AH=8 (get geometry)
	mov ecx,444d0000h	; magic2
	push dx
	mov edx,53490000h	; magic3 +
	pop dx			; ... drive number in DL
	mov ebx,3f4b0000h	; magic4
	int 13h			; BIOS DISK API
	shr eax,16		; ignore AX
	shr ebx,16		; ignore BX
	shr ecx,16		; ignore CX (geometry C/S)
	shr edx,16		; ignore DX (geometry H in DH)
	cmp ax,4d21h		; magic5
	jnz nomemdisk
	cmp cx,4d45h		; magic6
	jnz nomemdisk
	cmp dx,4944h		; magic7
	jnz nomemdisk
	cmp bx,4b53h		; magic8
	jnz nomemdisk

	push es
	push di
	les di,[es:di+12]
	call printcommandline
	pop di
	pop es

	; ES:DI now points to a data structure...
	; dw bytes, dw version, dd disk address in RAM,
	; dd size in sectors, dd pointer to commandline,
	; dd old int 13h vector, dd old int 15h vector
	; dw old [40h:13h] low memory size value

	stc			; return success
	ret

nomemdisk:
	clc			; return failure
	ret


		dw pcxt, pcat	; entries -2 and -1 of stringtable!
stringtable	dw xmem, amem, bmem, cmem

hello	db "@echo off"
	db 13,10,"$"

pcxt	db "echo 8086, cannot have$"	; 254
pcat	db "echo 80286, cannot have$"	; 255
xmem	db "echo scan found no$" 	; (in A B HDA)	; 0 (or 16)
amem	db "echo A: is$"		; 1 (or 17)
bmem	db "echo B: is$"		; 2 (or 18)
cmem	db "echo 1st harddisk is$"	; 3 (or 19)
memdisk	db " MEMDISK.",13,10,"$"

iswin	db "echo Win ("
winbits	db "32 bit) DOS box!",13,10,"$"			; modifier

WinCheck:
	push ax
	mov ax,1683h	; get current virtual machine number
	xor bx,bx	; preset to 0 if no Win at all running
	int 2fh
	or bx,bx
	pop ax
	jnz IsWin

	push ax
	mov ax,1600h	; Win32 install check
	int 2fh
	test al,7fh	; 0 / 80 no Win32, 1 Win2, -1 Win2, 3.. Win 3..
	pop ax
	jnz IsWin

	push ax
	push bx
	push cx
	mov ax,160ah	; Win3+ version check
	int 2fh
	or ax,ax	; 0 means Win3+ version check supported
	pop cx
	pop bx
	pop ax
	jnz WinChecked
	; version is BH.BL for CPU type CX

IsWin:	add al,16	; change errorlevel

	push ax
	mov ax,1600h	; do Win32 install check
	int 2fh
	or al,al	; major version number, or (Win2) +1/-1
	jnz WinIs32	; ah is minor version number.
	mov word [winbits],"16"	; not in enhanced / Win386 / Win32 mode
	; (the Win3.x 286 (standard mode) DOS extender has no version check)
WinIs32:
	mov dx,iswin
	mov ah,9
	int 21h
	pop ax

WinChecked:
	ret



printcommandline:	; string at ES:DI, 0 terminated, space separated
; http://people.cs.uchicago.edu/~gmurali/gui/comboot/getargs.c skips over
; leading and trailing spaces and treats args without = inside as special.
	push ax
	push bx
nextcl:	mov bx,setline+4	; target pointer
skipcl:	mov al,[es:di]
	inc di
	or al,al
	jz donecl
	cmp al,' '
	jz skipcl		; skip leading spaces

copycl:	mov [bx],al		; save useful byte
	inc bx
	mov al,[es:di]		; check next source byte
	inc di
	or al,al
	jz donecl
	cmp al,' '
	jnz copycl		; neither eof nor space? keep copying!

donecl:	cmp bx,setline+4
	jz nocl
	mov word [bx],0a0dh	; 13,10 (carriage return, linefeed)
	mov byte [bx+2],'$'	; end of string
	mov ah,9
	mov bx,dx		; save
	mov dx,setline
	int 21h			; print string to stdout
	mov dx,bx		; restore
nocl:	dec di			; rewind
	mov al,[es:di]		; the byte which ended THIS argument
	or al,al		; end of the whole thing?
	jnz nextcl		; otherwise repeat for NEXT argument
	pop bx
	pop ax
	ret

setline	db "SET foo=bar",13,10,"$"

